%QCORRECTION_IMS_AUTO computes FRC corrected for spurious correlations
%
% SYNOPSIS:
%  [resolution_corr, resolution, Q, frccurve_corr, frccurve, fit_sigmas, frc_numerator] = Qcorrection_ims_auto(in1, in2, params_in)
%
% PARAMETERS:
%  in1:         image of half the emitters
%  in2:         image of the other half of the emitters
%  params_in:   structure with the following fields
%                  sigmas:      estimated localization uncertainties (in SR pixels)
%                  minq:        maximum spatial frequency used for Q estimation
%                  maxq:        maximum spatial frequency used for Q estimation
%                  ps:          pixel size [pixel/nm]
%
% OUTPUT:
%  resolution_corr:     FRC resolution corrected for spurious correlations
%  resolution:          FRC resolution without correction
%  Q:                   
%  frccurve_corr:       FRC curve corrected for spurious correlations
%  frccurve:            FRC curve without correction
%  fit_sigmas:          Found sigma and std. dev. of sigma of spurious correlation component
%  frc_numerator:       Numerator of the FRC curve without correction

function varargout = Qcorrection_ims_auto(varargin)

if nargin<3
    error('Too few input variables');
else
    params_in = varargin{3};
    varargin{3} = struct2cell(varargin{3});
end

d = struct('menu','FRC resolution',...
           'display','FRC_Qcorrection6',...
           'inparams',struct('name',       {'in1',      'in2',          'params_in'},...
                             'description',{'Image 1',  'Image 2',      'Input parameters'},...
                             'type',       {'image',    'image',        'cellarray'},...
                             'dim_check',  {2,          2,              0},...
                             'range_check',{[],         [],             []},...
                             'required',   {1,          1,              0},...
                             'default',    {'in1',      'in2',          {[]}}...
                              ),...
'outparams',struct('name',{'resolution_corr','resolution_value','Q','frc_curve_corr','frc_curve','fit_sigmas','frc_numerator'},...
                              'description',{'Resolution (corrected)','Resolution (uncorrected)','Q','FRC curve (corrected)','FRC curve (uncorrected)','Fit parameters','FRC numerator'},...
                              'type',{'array','array','array','array','array','array','array'},...
                              'suppress',{1,1,1,1,1,1,1}...
                              )...
           );       
       
if nargin == 1
   s = varargin{1};
   if ischar(s) && strcmp(s,'DIP_GetParamList')
      varargout{1} = struct('menu','none');
      return
   end
end

try
   [in1, in2, ~] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error('Parameter parsing was unsuccessful.')
   end
end

if ~isa(in1,'dip_image') || ~isa(in2,'dip_image')
    error('dipimage inputs required.');
end

% Check input parameters
if isfield(params_in,'sigmas')
    sigmas = params_in.sigmas;
else
    warning('frc_Qcorrection6:nosigmas','No localization uncertainties specified');
    sigmas = 1;
end

if isfield(params_in,'maxq')
    maxq = params_in.maxq;
    if maxq > 0.5
        maxq = 0.5;
    end
else
    maxq = 0.45;
end

if isfield(params_in,'minq')
    minq = params_in.minq;
elseif isfield(params_in,'d_plateau')
    d_plateau = params_in.d_plateau;
    minq = maxq-0.5*d_plateau;
else
    minq = 0;    
end
minq(minq<0) = 0;

if isfield(params_in,'ps')
    ps = params_in.ps;
else
    ps = 1;
end

if isfield(params_in,'lastvals')
    last_vals = params_in.lastvals;
    skip_pfit = 1;
else
    skip_pfit = 0;
end

n_sigmas = 1e4; % Hard coded maximum number of localizations used for plateau fit

%% Prepare calculation of Q
sz = imsize(in1);
mask = frc_tapermask(max(sz),[1 sz(1)],[1 sz(2)],1);
sz = max(sz);
in1 = mask*extend(in1,sz*[1 1],'topleft',0);
in2 = mask*extend(in2,sz*[1 1],'topleft',0);

Qnorm = (1/mean(in1)+1/mean(in2));

%% Calculate FRC numerator and denominator
in1 = ft(in1);
in2 = ft(in2);
frcnum = double(real(radialmean(in1.*conj(in2))));
frcdenom = double(sqrt(real(radialmean(abs(in1).^2).*radialmean(abs(in2).^2))));

q = (0:(length(frcnum)-1))./sz;             % Spatial frequencies
q_sel = (q>=minq)&(q<=maxq);                % Domain in q for plateau fitting

%% Obtain initial estimate of Q plateau height and decay
if ~skip_pfit
    % Use sigmas from input as initial plateau
    if length(sigmas)<n_sigmas
        exp_decay = mean(exp(-4*pi^2*(sigmas.^2)*(q.^2)),1).*sinc(q).^2;
    else
        sigmas_sel = sigmas(randperm(length(sigmas),n_sigmas));
        exp_decay = mean(exp(-4*pi^2*(sigmas_sel.^2)*(q.^2))).*sinc(q).^2;
    end
    
    smoothlog = im2mat(medif(mat2im(Qnorm*frcnum./exp_decay),5));
    smoothlog = log(abs(smoothlog));
    p_fit = fit(q(q_sel)',smoothlog(q_sel)','poly2','Upper',[Inf eps Inf],'Lower',[-Inf -eps -Inf]);
    
    % Starting value for estimated sigma
    Q_start = exp(p_fit.p3);    
    s_start = double(real(sqrt(mean(sigmas.^2)-p_fit.p1/(4*pi^2))));
    last_vals = [Q_start s_start std(sigmas)];
end    
    
%% Estimate spurious component by promoting plateauness

[X1,V1] = fminsearch(@(c) plateauness(q(q_sel),Qnorm*frcnum(q_sel),c),last_vals);
[X2,V2] = fminsearch(@(c) plateauness(q(q_sel),Qnorm*frcnum(q_sel),c),last_vals.*(0.5+1.0*rand(1,3)));

if V1 <= V2
    X = X1;
else
    X = X2;
end

Q = X(1);

frcnum_corr = Q/Qnorm*sinc(q).^2.*exp(-4*pi^2*X(2)^2*q.^2./(1+8*pi^2*X(3)^2*q.^2))./sqrt(1+8*pi^2*X(3)^2*q.^2);

% fprintf('Estimated Q: %2.2f.\n',Q);
% fprintf('Estimated sigma: %2.2f +- %2.2f nm.\n',abs(X(2))*ps,abs(X(3))*ps);

%% Correct for spurious correlations and calculate resolutions

frccurve = frcnum./frcdenom;
frccurve_corr = (frcnum-frcnum_corr)./(frcdenom +frcnum_corr);

resolution = frctoresolution(frcnum./frcdenom,sz);
resolution_corr = frctoresolution((frcnum-frcnum_corr)./(frcdenom +frcnum_corr),sz);

%% Generate outputs

varargout{1} = resolution_corr;
varargout{2} = resolution;
varargout{3} = Q;
varargout{4} = frccurve_corr;
varargout{5} = frccurve;
varargout{6} = [X(2) X(3)];
varargout{7} = Qnorm*frcnum;

%% Plot results
figure
subplot(1,3,1)
hold on
plot(q/ps,frccurve,'r');
plot(q/ps,frccurve_corr,'b')
plot([0 1/ps],[0 0],'k');
xlim([0 0.5/ps])
ylim([-0.2 1.2])
xlabel('spatial freq [nm^{-1}]')
ylabel('FRC')
legend('Normal','Corrected')
hold off
subplot(1,3,2)
hold on
plot(q(frcnum>0)/ps,Qnorm*frcnum(frcnum>0),'r');
plot(q/ps,Qnorm*frcnum_corr,'k:','LineWidth',2)
plot(q(frcnum>0)/ps,Qnorm*abs(frcnum(frcnum>0)-frcnum_corr(frcnum>0)),'b');
set(gca,'YScale','log')
hold off
xlim([0 0.5/ps])
xlabel('spatial freq [nm^{-1}]')
ylabel('FRC')
legend('FRC numerator','Spurious component','Corrected numerator')
subplot(1,3,3)
hold on
plot(q/ps,frcnum./frcnum_corr);
plot([0 0.5/ps],[1 1],'k')
hold off
xlim([0 0.5/ps])
ylim([0 2])
xlabel('spatial freq [nm^{-1}]')
ylabel('FRC numerator / Spurious component')
end

%% Optimization cost function
function V = plateauness(q,frcnum,c)
frcnum_noisevar = 0.1;

Q_tmp = c(1);
meansig = c(2);
stdsig = c(3);

stdfac = 1+8*pi^2*stdsig^2*(q.^2);
model_decay = exp(-4*pi^2*meansig^2*(q.^2)./stdfac)./sqrt(stdfac);

diff_plateau = frcnum./(Q_tmp*model_decay.*sinc(q).^2)-1;
V = sum((1-exp(-diff_plateau.^2/(frcnum_noisevar^2))));
end
