%Qcount   Estimate the number of localizations per labeled site
%
% SYNOPSIS:
%  [Q,t,N,M,Fit_switching,Fit_sigmas,FRC_output,ROI_output] = Qcount(localizations,FOV_size,zoomfactor,autoQplateau,Qparameters,pixelsize,stoich_calibration)
%
% OUTPUT:
%  Q                   
%  t:               Time points for which Q is computed
%  N:               Cumulative localizations until time t
%  M:               Estimated localizations per labeled site
%  Fit_switching:   Switching model parameters obtained from a fit to Q(t);
%  Fit_sigmas:      Estimated effective localization precisions (sigma) obtained from the plateau finding. NB: Results given in nm.
%  FRC_output:      Outcomes of the FRC resolution computation. NB: Resolution is in nm, spatial frequencies in 1/nm.
%  ROI_output:      Selected ROI
%
% PARAMETERS:
%  localizations:       Nx3 array of localizations, formatted as [x,y,t]
%  FOV_size:            Maximum allowed values for (x,y) in the localizations array
%  zoomfactor:          Superresolution pixels / unit length in (x,y) in localizations array
%  autoQplateau:        Estimate the Q plateau automatically? [0/1]
%  Qparameters:         Structure with input parameters for plateau fitting; depends on autoQplateau
%                           autoQplateau = 0: minq, maxq, sigma_mean, sigma_std, Qpoints (optional), reps (optional), nblocks (optional), noROI (optional)
%                           autoQplateau = 1: minq, maxq, sigmas (optional), Qpoints (optional), reps (optional), nblocks (optional), noROI (optional)        
%  pixelsize:           nm / unit length in (x,y) in localizations array (optional)
%  stoich_calibration:  Found values [mu,S] in the stoichiometry calibration. If S is not specified, then S = mu/(1-exp(-mu)).

function varargout = Qcount(varargin)

% First extract structure with parameters for Q finding
if length(varargin)<5
    error('Too few input arguments');
else
    params_in = varargin{5};
    
    if ~isstruct(params_in)
        error('Parameters for Q finding not specified.');
    end
    
    varargin{5} = {};
end

% Obtain remaining inputs
d = struct('menu','FRC resolution',...
           'display','Q-counting',...
           'inparams',struct('name',       {'localizations',        'sz',                   'zoomfactor',   'autoQplateau',          'params_in',         'Pixel size',     'stoich_cal'},...
                             'description',{'Localizations',        'Image dimensions',     'Zoom',         'Plateau finding method','Input parameters',  'Pixel size (nm)','Stoichiometry values'},...
                             'type',       {'array',                'array',                'array',        'boolean',               'cellarray',         'array',          'array'},...
                             'dim_check',  {{[],[-1 3],[-1 7],[-1 8]},{[],0,[1 2],[2 1]},   {[],0},         0,                       0,                   0,                {[],0,[1 2],[2 1]}},...
                             'range_check',{'R',                    'N+',                   [eps Inf],      [],                      [],                  [eps Inf],        'R+'},...
                             'required',   {1,                      0,                      0,              0,                       1,                   0,                0},...
                             'default',    {[],                     [],                     1,              1,                       {[]},                1,              '[]'}...
                              ),...
           'outparams',struct('name',{'Q','t','N','M','fit_switching','fit_sigmas','frc_outputs','roi_sel'},...
                              'description',{'Q(t)','Time points t','N(t)','M(t)','Fit outputs (switching)','Fit outputs (plateau)','FRC outputs','ROI'},...
                              'type',{'array','array','array','array','array','array','array','array'},...
                              'suppress',{1,1,1,1,1,1,1,1}...
                              )...
           );       
       
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      varargout{1} = d;
      return
   end
end

try
   [coords,sz,zm,autoQplateau,~,pixelsize,stoich_cal] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error('Parameter parsing was unsuccessful.')
   end
end


if isempty(sz)
    sz = ceil(max(max(coords(:,1:2))));
end

if numel(sz) == 1
    sz = [sz sz];
end

if isempty(zm)
    zm = 10;
end

%% Select ROI

if ~(isfield(params_in,'noROI') && params_in.noROI)
    selROI = questdlg('Select ROI for quantification?','ROI selection','Yes','No','Cancel','No');

    switch selROI
        case 'Yes'
            [coords_sel,roi_sel] = selectlocalizations(coords,zm);
            coords_sel(:,1) = coords_sel(:,1) - roi_sel(1,1);
            coords_sel(:,2) = coords_sel(:,2) - roi_sel(1,2);
            coords = coords_sel;
            sz = floor(roi_sel(2,:));
            clear coords_sel
        case 'No'
            % Resume
        case 'Cancel'
            error('Q counting aborted');
        otherwise
            % Resume
    end
end

%% Suppress questions about plateau quality

if (isfield(params_in,'noCheck') && params_in.noCheck)
    noCheck = 1;
else
    noCheck = 0;
end

%% Extract parameters for Q fitting

if isfield(params_in,'d_plateau')
    d_plateau = params_in.d_plateau;
else
    d_plateau = [];
end

if isfield(params_in,'maxq')
   maxq = params_in.maxq;
else
   maxq = 0.45; % default
end

if isfield(params_in,'minq')
    minq = params_in.minq;
elseif isfield(params_in,'d_plateau')
    d_plateau = params_in.d_plateau;
    minq = maxq-0.5*d_plateau;
else
    minq = 0;    
end
minq(minq<0) = 0;

if autoQplateau
    if isfield(params_in,'sigmas')
       sigmas = double(params_in.sigmas); % Overwritten for 
    else
       sigmas = coords(:,5); % default
    end
else
    sigma_mean = params_in.sigma_mean;

    if isfield(params_in,'sigma_std')
        sigma_std = params_in.sigma_std;
    else
        sigma_std = 0;
    end
end

if isfield(params_in,'reps')
    reps = params_in.reps;
else
    reps = 1;
end

if isfield(params_in,'nblocks')
    nblocks = params_in.nblocks;
else
    nblocks = 20;
end
    
if isfield(params_in,'Qpoints')
    if isscalar(params_in.Qpoints) && (params_in.Qpoints > 0)
        Qpoints = params_in.Qpoints;
    else
        Qpoints = 20; % default
    end
else
   Qpoints = 20; % default
end




%% Prepare Q computation

tmax = max(coords(:,3));
t = linspace(0,tmax,Qpoints+1); t = t(2:end); t = t';
Qs = zeros(Qpoints,reps);
N = zeros(Qpoints,1);
R_store = zeros(Qpoints,reps);

if autoQplateau
    iter_params = struct('sigmas',zm*sigmas,'maxq',maxq,'minq',minq,'ps',pixelsize/zm,'noCheck',noCheck);
    sigmas_est = zeros(Qpoints,reps,2); % Estimated {sigma_m,dsigma} for automatic plateau finding
else
    iter_params = struct('sigma_mean',zm*sigma_mean,'sigma_std',zm*sigma_std,'maxq',maxq,'minq',minq,'ps',pixelsize/zm,'noCheck',noCheck);
end

%% Compute Q for specified time points

for nn = 1:reps
    frcnums = cell(length(t),1);
    fprintf(' -- Estimating Q(t), iteration %d/%d --\n',nn,reps)
    for tt = length(t):-1:1
        fprintf('Time point %d/%d\n',(length(t)-tt+1),length(t));
        coords_tmp = coords(coords(:,3)<=t(tt),:);
               
        % Split data in time blocks
        coords_tmp(:,3) = ceil(coords_tmp(:,3)/max(coords_tmp(:,3))*nblocks);       % Go from time stamps to block numbers of the localizations                                      
        blocksel = randperm(nblocks,ceil(nblocks/2));                               % Block numbers in the first half data set
        s = ismember(coords_tmp(:,3),blocksel);                                     % Localizations in the first half data set

        % Bin images into 2 half data images
        in1 = showlocalizations(coords_tmp(s,:),sz,zm);
        in2 = showlocalizations(coords_tmp(~s,:),sz,zm);

        if autoQplateau
            % Q fitting with automatic plateau finding
            if (nn == 1) && (tt == length(t))
                % For the first Q-fit, check if the found solution is acceptable
                check_cycle = 1;
                while check_cycle
                    [R,~,Q,frc_out,frc_orig,vals_tmp,frcnum_tmp] = Qcorrection_ims_auto(in1,in2,iter_params);
                    [check_cycle,iter_params] = manualparamokay(autoQplateau,iter_params,Q,vals_tmp);
                end
            else
                [R,~,Q,~,~,vals_tmp,frcnum_tmp] = Qcorrection_ims_auto(in1,in2,iter_params);
                close
            end
                 
            sigmas_est(tt,nn,:) = vals_tmp; % Fitted parameters of localization error distribution
            iter_params.lastvals = [Q*((tt-1)/tt) vals_tmp];                                % Use values from previous iteration for next iteration
            
        else
            % Q fitting with manual plateau specification
            if (nn == 1) && (tt == length(t))
                % For the first Q-fit, check if the found solution is acceptable
                check_cycle = 1;
                while check_cycle
                    [R,~,Q,frc_out,frc_orig,frcnum_tmp] = Qcorrection_ims_manual(in1,in2,iter_params);
                    [check_cycle,iter_params] = manualparamokay(autoQplateau,iter_params);
                end
            else
                [R,~,Q,~,~,frcnum_tmp] = Qcorrection_ims_manual(in1,in2,iter_params);
                close
            end
            
            iter_params.lastvals = Q*((tt-1)/tt);	% Use values from previous iteration for next iteration
        end
        
        if tt == 1
            iter_params =  rmfield(iter_params,'lastvals');
        end
        

        % Write outputs into container variables
        if nn == 1
            % Compute cumalative localizations
            N(tt) = size(coords_tmp,1);
        end
        frcnums{tt} = frcnum_tmp;
        R_store(tt,nn) = R;
        Qs(tt,nn) = Q;
        
    end
    
    % Remove prior parameter estimates to go into new iteration unbiased
    if isfield(iter_params,'lastvals')
        iter_params = rmfield(iter_params,'lastvals');
    end
    
end

%% Return Q estimation outputs

varargout{1} = Qs;
varargout{2} = t;
varargout{3} = N;

if autoQplateau
    varargout{6} = struct('sigma_mean',squeeze(sigmas_est(:,:,1))/zm*pixelsize,'sigma_std',squeeze(sigmas_est(:,:,2))/zm*pixelsize);
else
    varargout{6} = struct('sigma_mean',sigma_mean*pixelsize,'sigma_std',sigma_std*pixelsize);
end

q = (0:length(frc_out)-1)/max(imsize(in1))/iter_params.ps;
varargout{7} = struct('frc_Qcorrected',frc_out,'frc_uncorrected',frc_orig,'resolution_mean',mean(R_store,2),'resolution_std',std(R_store,[],2),'spatial_freq',q);

%% Estimate the number of localizations per site
if numel(stoich_cal) == 1
    % Only mu specified
    [QNmodel_params,M,N_model,Q_model] = Qfit(t,N,Qs,stoich_cal(1));
elseif numel(stoich_cal) == 2
    % Mu and S specified
    [QNmodel_params,M,N_model,Q_model] = Qfit(t,N,Qs,stoich_cal(1),stoich_cal(2));
else
    % No stoichiometry parameters specified, so Q if fit for mu instead
    [QNmodel_params,M,N_model,Q_model] = Qfit(t,N,Qs);
end

%% Return counting outputs
varargout{4} = M;
% varargout{5} = QNmodel_params(1:3);
varargout{5} = struct('M_inf',QNmodel_params(1),'k_bl',1/QNmodel_params(2),'mu',QNmodel_params(3));

%% Return ROI
if strcmp(selROI,'Yes')
    varargout{8} = struct('ROI_topleft',roi_sel(1,:),'ROI_size',sz);
else
    varargout{8} = struct('ROI_size',sz);
end

%% Plot counting outputs
figure
subplot(1,3,1)
hold on
plot([0;t],[0;N_model],'k:');
plot(t,N,'kx');
xlim([0 max(t)])
xlabel('t [frames]')
ylabel('Cumulative localizations N(t)');
legend('Model fit','Data','Location','SouthEast')

subplot(1,3,2)
hold on
plot([0;t],[0;Q_model],'k:');
plot(t,Qs,'kx');
xlim([0 max(t)])
xlabel('t [frames]')
ylabel('Spurious correlation parameter Q(t)');
legend('Model fit','Estimated from FRC','Location','SouthEast')

subplot(1,3,3)
plot([0;t],[0;M],'k');
xlim([0 max(t)])
xlabel('t [frames]')
ylabel('Localizations per labeled site');

end

% Function to prompt user to check Q-plateau and adjust fitting parameters
% if necessary
function varargout = manualparamokay(varargin)
    
    autoQplateau = varargin{1};
    iter_params = varargin{2};
    ps = iter_params.ps;
    
    if iter_params.noCheck
        % Skip plateau check if desired
        varargout{1} = 0;
        varargout{2} = iter_params;
        return
    end
    
    % Ask if the found plateau is acceptable
    prompt_paramokay = questdlg('Is the fit to the FRC numerator okay?','Plateau check','Yes','No','Cancel','Yes');       
        
    switch prompt_paramokay
        case 'Yes'
            varargout{1} = 0;
            close
        case 'No'
            % Offer to change fit parameters
            if autoQplateau
                Q = varargin{3};
                vals_tmp = varargin{4};
                answer = {num2str(vals_tmp(1)*ps),num2str(vals_tmp(2)*ps),num2str(iter_params.minq),num2str(iter_params.maxq)};
                answer = inputdlg({'Mean sigma [nm]','Std. dev. of sigma [nm]','Min. freq.','Max. freq'},'Change input parameters',1,answer);
                iter_params.lastvals = [Q,str2double(answer{1})/ps,str2double(answer{2})/ps];

            else
                answer = {num2str(iter_params.sigma_mean*ps),num2str(iter_params.sigma_std*ps),num2str(iter_params.minq),num2str(iter_params.maxq)};
                answer = inputdlg({'Mean sigma [nm]','Std. dev. of sigma [nm]','Min. freq.','Max. freq'},'Change input parameters',1,answer);
                iter_params.sigma_mean = str2double(answer{1})/ps;
                iter_params.sigma_std = str2double(answer{2})/ps;
            end
            iter_params.minq = str2double(answer{3});
            iter_params.maxq = str2double(answer{4});
            
            varargout{1} = 1;
            close
        case 'Cancel'
            error('Q counting aborted');
        otherwise
            varargout{1} = 0;
            close
    end   
    
    varargout{2} = iter_params;     % Output parameter structure, containing the new fit parameters
end


