%Qfit Fit Q(t) and N(t) to obtain the number of localizations per site M(t)
%
% SYNOPSIS:
% function [model_params M N_model Q_model] = Qfit(t,N,Q,mu,S)
%
% OUTPUT:
%  model_params:    [M_inf,t_bl,mu,N_inf]
%  M:               Number of localizations per site
%                   = S*M_inf*(1-exp(-t/t_bl))
%  N_model:         Fitted model to the cumulative localizations N
%                   = N_inf*(1-exp(-t/t_bl)
%  Q_model:         Fitted model to the spurious correlation parameter Q
%                   = 2*(M_inf-1)*(1-(t/t_bl)./(exp(t/t_bl)-1))+M_inf*mu*(1-exp(-t/t_bl));

function varargout = Qfit(varargin)

d = struct('menu','FRC resolution',...
           'display','Q fitting',...
           'inparams',struct('name',       {'t',                'N',                'Q',            'mu',   'S'},...
                             'description',{'Times',            'Cumulative locs',  'Q',            'mu',   'S'},...
                             'type',       {'array',            'array',            'array',        'array','array'},...
                             'dim_check',  {-1,                 -1,                 {-1,[-1 -1]},   {[],0}, {[],0}},...
                             'range_check',{'R+',               'R+',               'R',            'R+',   'R+'},...
                             'required',   {1,                  1,                  1,              0,       0},...
                             'default',    {[],                 [],                 [],             '[]',    1}...
                              ),...
           'outparams',struct('name',{'frc_out'},...
                              'description',{'FRC curve'},...
                              'type',{'array'}...
                              )...
           );       
       
if nargin == 1
   s = varargin{1};
   if ischar(s) && strcmp(s,'DIP_GetParamList')
      varargout{1} = struct('menu','none');
      return
   end
end

try
   [t,N,Q,mu_in,S] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error('Parameter parsing was unsuccessful.')
   end
end

% Check size of t and N
if size(t,1)~=size(Q,1)
    t = t';
end

if size(N,1)~=size(Q,1)
    N = N';
end

%% Fit Q
Q_in = median(Q,2);

w = 1;                  % Weight of N error / Weight of Q_error

if isempty(mu_in)
    X = fminsearch(@(c) Qfit_optimfunc_nomu(t,N,Q_in,c,w),[5,1e4,1,N(end)],optimset('MaxFunEvals',1000));
    params_out = X(1:4);
else
    X = fminsearch(@(c) Qfit_optimfunc_mu(t,N,Q_in,mu_in,c,w),[5,1e4,N(end)],optimset('MaxFunEvals',1000));
    params_out = [X(1:2), mu_in, X(3)];
end    

%% Prepare outputs
params_out(1) = X(1)^2+1;

% Check if the number of labels per marker is provided
if nargin<5
    % Assume S = mu/(1-exp(-mu))
    S = params_out(3)/(1-exp(-params_out(3)));  
end

%% Create outputs
varargout{1} = params_out;
varargout{2} = S*params_out(1)*(1-exp(-t/params_out(2)));
varargout{3} = N_model(t,X(end),params_out(2));
varargout{4} = Q_model(t,params_out(1),params_out(2),params_out(3));

function out = Qfit_optimfunc_nomu(t,N,Q,c,w)    

c(1) = c(1)^2+1;

% Compute weight for errors in N and Q
w_N = w.*(1./N_model(t,c(4),c(2)))/N_model(t(end),c(4),c(2));
% w_Q = Q_model(t,c(1),c(2),c(3)).^-2
w_Q = Q_model(t(end),c(1),c(2),c(3)).^-2;

% Weighted sum of the errors in the model for Q(t) and N(t)
out = sum(w_N.*(N-N_model(t,c(4),c(2))).^2)+sum(w_Q.*(Q-Q_model(t,c(1),c(2),c(3))).^2);

function out = Qfit_optimfunc_mu(t,N,Q,mu_in,c,w)    

c(1) = c(1)^2+1;

% Compute weight for errors in N and Q
w_N = w.*(1./N_model(t,c(3),c(2)))/N_model(t(end),c(3),c(2));
% w_Q = Q_model(t,c(1),c(2),mu_in).^-2
w_Q = Q_model(t(end),c(1),c(2),mu_in).^-2;

out = sum(w_N.*(N-N_model(t,c(3),c(2))).^2)+sum(w_Q.*(Q-Q_model(t,c(1),c(2),mu_in)).^2);

function out = N_model(t,N_inf,t_bl)
out = N_inf*(1-exp(-t/t_bl));

function out = Q_model(t,M_inf,t_bl,mu_val)
out = 2*(M_inf-1)*(1-(t/t_bl)./(exp(t/t_bl)-1))+M_inf*mu_val*(1-exp(-t/t_bl));

