%SHOWLOCALIZATIONS   Create image of localizations
%
% SYNOPSIS:
%   out = showlocalizations(localizations,size,zoomfactor,method,unc_column)
%
% PARAMETERS:
%   size
%      Size in x and y-direction of input localizations [CCD pixels]
%   zoomfactor
%      Superresolution pixels per unit distance in the positions list
%   method
%      Rendering method: bin (default), jitter, blobs
%   unc_column
%      column of localization uncertainty
%
% DEFAULTS:
%   size : image edges correspond to maximum x- and y-coordinates
%   zoomfactor = 1
%   method = 'bin'
%   method_parameter = 5
%
% NOTES:
%  Columns of positions are assumed to have the same dimensionality.
%  Localizations with x or y-values smaller than -0.5 are discarded.

% (C) Copyright 2014               Quantitative Imaging Group
%     All rights reserved          Faculty of Applied Physics
%                                  Delft University of Technology
%                                  Lorentzweg 1
%                                  2628 CJ Delft
%                                  The Netherlands
% Robert Nieuwenhuizen, Feb 2014


function out = showlocalizations(varargin)

% Hard coded parameter
NumJitter = 50;
MaxBlobsCPU = 1e4;

d = struct('menu','FRC resolution',...
           'display','Display localizations',...
           'inparams',struct('name',       {'positions',        'sz',                   'zm',       'vis_meth',                 'unc_column'},...
                             'description',{'Localizations',    'Size of input data',   'Zoom',     'Rendering method',         'Column of localization uncertainty'},...
                             'type',       {'array',            'array',                'array',    'option',                   'array'},...
                             'dim_check',  {{[],[-1 -1]},       {[],0,[1 2],[2 1]},     {[],0},     0,                          0},...
                             'range_check',{'R',                'N+',                   [eps Inf],  {'bin','jitter','blobs'},   'N+'},...
                             'required',   {0,                  0,                      0,          0,                          0},...
                             'default',    {[],                 [],                     1,          'bin',                      5}...
                              ),...
           'outparams',struct('name',{'out'},...
                              'description',{'Superresolution image'},...
                              'type',{'array'}...
                              )...
           );       

if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      out = d;
      return
   end
end

try
   [coords,sz,zm,vis_meth,unc_column] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error('Parameter parsing was unsuccessful.')
   end
end       

% Check that an output image size can be defined
if isempty(coords) && isempty(sz)
        error('No output image size can be defined.')
end

if ~(isempty(coords) || (size(coords,2)>=2))
    error('Localization data need to contain at least two columns.');
end

% Prepare variables
if isempty(zm)
    zm = 1;
end

if isempty(sz)
   sz = ceil(max(coords(:,1:2),[],1)+0.5); 
end

if isscalar(sz)
    sz = [sz sz];
end

sz = single(ceil(sz*zm));

if isempty(coords)
    out = newim(sz);
    return
end

% Offset and rescale positions
coords = single(coords);
coords(:,1:2) = coords(:,1:2) + 0.5;
coords(:,1:2) = coords(:,1:2)*zm;

% Filter localizations outside the FOV
keep = coords(:,1)>=0;
keep = keep & coords(:,2)>=0;
keep = keep & coords(:,1)<=sz(1);
keep = keep & coords(:,2)<=sz(2);
coords = coords(keep,:); 

switch vis_meth   
    case 'bin'
        % Histogram image of localizations
        out = cHistRecon(sz(1),sz(2),coords(:,1),coords(:,2),0)';
        out = mat2im(out);
    case 'jitter'
        % Histogram image of jittered localizations
        coords(:,unc_column) = coords(:,unc_column)*zm;
        out = uint16(zeros(sz));
        for nn = 1:NumJitter
            coords_tmp(:,1:2) = coords(:,1:2) + (coords(:,unc_column)*[1 1]).*randn(size(coords(:,1:2)));
            out = out+cHistRecon(sz(1),sz(2),coords_tmp(:,1),coords_tmp(:,2),0)';
        end
        out = single(out)/NumJitter;
        out = mat2im(out);
    case 'blobs'
        % Gaussian rendering of localizations
        coords(:,unc_column) = coords(:,unc_column)*zm;

        out = newim(sz);
        Niter = ceil(size(coords,1)/MaxBlobsCPU);

        for nn = 1:Niter
            st = (nn-1)*Niter+1;
           if nn < Niter
               en = nn*Niter;                   
           else
               en = size(coords,1);
           end
           % Add blobs to image out. N.B. Gaussianblob appears to be slow
           % in the current release of DIPimage (as of 20/03/2015). Use the
           % version of Gaussian blob in the private directory instead.
           out = gaussianblob(out,coords(st:en,1:2),coords(st:en,unc_column),ones(en-st+1,1),'spatial',3);
        end            
end
